@@ -26,7 +26,7 @@
 #endif
 
 #if !defined(CLASP_VERSION)
-#define CLASP_VERSION "3.0.1"
+#define CLASP_VERSION "3.0.3"
 #endif
 #if !defined(CLASP_LEGAL)
 #define CLASP_LEGAL \
@@ -106,6 +106,7 @@ struct ExpectedQuantity {
 //! Provides a simplified interface to the services of the clasp library.
 class ClaspFacade : public EventHandler {
 	struct SolveImpl;
+	struct SolveStrategy;
 public:
 	//! Result of a solving step.
 	struct Result {
@@ -213,41 +214,79 @@ public:
 	//! Solves the current prepared problem.
 	/*!
 	 * \pre !solving()
+	 * \param handler An optional event handler that is notified on each model and
+	 *                once after the solve operation has completed.
+	 * \note If ok() is false or result().unsat() is true prior to
+	 *       calling solve(), i.e. the current step is already complete,
+	 *       the solve operation is not started and no events are generated.
 	 */
 	Result             solve(Clasp::EventHandler* onModel = 0);
 #if WITH_THREADS
-	//! A type for accessing the result of an asynchronous solve operation.
+	struct AsyncSolve;
+	//! A type for accessing the result(s) of an asynchronous solve operation.
 	class AsyncResult {
 	public:
-		explicit AsyncResult(SolveImpl& x);
 		typedef  StepReady Ready;
+		explicit AsyncResult(SolveImpl& x);
+		~AsyncResult();
+		AsyncResult(const AsyncResult&);
+		AsyncResult& operator=(AsyncResult temp)            { swap(*this, temp); return *this; }
+		friend void swap(AsyncResult& lhs, AsyncResult& rhs){ std::swap(lhs.state_, rhs.state_); }
 		// blocking operations
-		//! Waits until the result is ready and returns it.
+		//! Waits until a result is ready and returns it.
 		Result get()              const;
-		//! Waits until the result is ready.
+		//! Waits until a result is ready.
 		void   wait()             const;
-		//! Waits for at most sec seconds.
+		//! Waits for a result but for at most sec seconds.
 		bool   waitFor(double sec)const;
 		//! Tries to cancel the active async operation.
 		bool   cancel()           const;
 
 		// non-blocking operations
-		//! Tests whether the result is ready.
+		//! Tests whether a result is ready.
 		bool   ready()            const;
-		//! Returns the result in r if it is ready.
+		//! Returns the active result in r if it is ready.
 		bool   ready(Result& r)   const;
-		//! Tests whether the operation was interrupted and if so returns the interruption signal.
+		//! Tests whether the asynchronous operation was interrupted and if so returns the interruption signal.
 		int    interrupted()      const;
-		//! Tests whether the asynchronous result is ready and has a stored exception.
+		//! Tests whether a result is ready and has a stored exception.
 		bool   error()            const;
+		//! Tests whether the asynchronous operation is still active.
+		bool   running()          const;
+
+		// model iterator operations
+		// Example:
+		//  for (AsyncResult it = facade.startSolveAsync(); !it.end(); it.next()) {
+		//    printModel(it.model());
+		//  }
+		//! Returns true if the iterator is exhausted (blocking).
+		bool               end()    const;
+		//! Advances the iterator by kicking off the search for the next model (non-blocking).
+		void               next()   const;
+		//! Returns the current model provided that end() is false (non-blocking).
+		const Model&       model()  const;
 	private:
-		SolveImpl* state_;
+		AsyncSolve* state_;
 	};
 	//! Asynchronously solves the current prepared problem.
 	/*!
 	 * \pre !solving()
+	 * \see solve(Clasp::EventHandler*)
+	 * \note The optional event handler is notified in the context of the
+	 *       asynchronous operation.
 	 */
 	AsyncResult        solveAsync(Clasp::EventHandler* handler = 0);
+
+	//! Asynchronously solves the current prepared problem signaling models one by one.
+	/*!
+	 * The function behaves similar to solveAsync() but yields models
+	 * one by one via the returned result object.
+	 * \pre !solving()
+	 * \note It is the caller's responsibility to finish the solve operation, 
+	 * either by extracting models until AsyncResult::end() returns true, or
+	 * by calling AsyncResult::cancel().
+	 */
+	AsyncResult        startSolveAsync();
 #endif
 	//! Returns whether the problem is still valid.
 	bool               ok()                  const { return program() ? program()->ok() : ctx.ok(); }
@@ -270,11 +309,10 @@ public:
 	ExpectedQuantity   getStat(const char* path)const;
 	const char*        getKeys(const char* path)const;
 private:
-	typedef SingleOwnerPtr<ProgramBuilder>  BuilderPtr;
-	typedef SingleOwnerPtr<Asp::LpStats>    LpStatsPtr;
-	typedef SingleOwnerPtr<Enumerator>      EnumPtr;
-	typedef SingleOwnerPtr<SolveImpl>       SolvePtr;
-	typedef SingleOwnerPtr<Summary>         SummaryPtr;
+	typedef SingleOwnerPtr<ProgramBuilder> BuilderPtr;
+	typedef SingleOwnerPtr<Asp::LpStats>   LpStatsPtr;
+	typedef SingleOwnerPtr<SolveImpl>      SolvePtr;
+	typedef SingleOwnerPtr<Summary>        SummaryPtr;
 	ExpectedQuantity getStatImpl(const char* path, bool keys)const;
 	ExpectedQuantity getStat(const SharedContext& ctx, const char* key, bool accu, const Range<uint32>& r) const;
 	void   init(ClaspConfig& cfg, bool discardProblem);
@@ -286,10 +324,8 @@ private:
 	bool   onModel(const Solver& s, const Model& m);
 	ClaspConfig* config_;
 	BuilderPtr   builder_;
-	EnumPtr      enum_;
 	LpStatsPtr   lpStats_;
 	SolvePtr     solve_;
-	EventHandler*stepHandler_;
 	SummaryPtr   accu_;
 	LitVec       assume_;
 	Summary      step_;
