@@ -93,6 +93,7 @@ public:
 
 	//! Options for the Asp-Preprocessor.
 	struct AspOptions {
+		static const uint32 MAX_EQ_ITERS = static_cast<uint32>( (1u<<26)-1 );
 		AspOptions() : erMode(mode_native), iters(5), noSCC(0), dfOrder(0), backprop(0), normalize(0), suppMod(0), noGamma(0) {}
 		AspOptions& iterations(uint32 it)   { iters   = it;return *this;}
 		AspOptions& depthFirst()            { dfOrder = 1; return *this;}
@@ -372,7 +373,7 @@ public:
 	typedef std::pair<EdgeIterator, EdgeIterator> EdgeRange;
 	typedef std::pair<HeadIter, HeadIter>         HeadRange;
 	const AspOptions& options()    const { return opts_; }
-	bool       hasConflict()       const { return atoms_[0]->literal() == posLit(0); }
+	bool       hasConflict()       const { return getFalseAtom()->literal() == posLit(0); }
 	bool       ok()                const { return !hasConflict() && ProgramBuilder::ok(); }
 	PrgAtom*   getAtom(Var atomId) const { return atoms_[atomId]; }
 	PrgHead*   getHead(PrgEdge it) const { return it.isAtom() ? (PrgHead*)getAtom(it.node()) : (PrgHead*)getDisj(it.node()); }
@@ -398,7 +399,7 @@ public:
 	PrgAtom*   mergeEqAtoms(PrgAtom* a, Var rootAtom);
 	PrgBody*   mergeEqBodies(PrgBody* b, Var rootBody, bool hashEq, bool atomsAssigned);
 	bool       propagate()               { return propagate(options().backprop); }
-	void       setConflict()             { atoms_[0]->setLiteral(posLit(0)); }
+	void       setConflict()             { getFalseAtom()->setLiteral(posLit(0)); }
 	RuleState& ruleState()               { return ruleState_; }
 	void       addMinimize();
 	// ------------------------------------------------------------------------
@@ -413,9 +414,8 @@ private:
 	LogicProgram(const LogicProgram&);
 	LogicProgram& operator=(const LogicProgram&);	
 	typedef PodVector<Rule*>::type RuleList;
-	typedef std::multimap<uint32, uint32>   BodyIndex; // hash -> bodies[offset]
-	typedef std::multimap<uint32, uint32>   DisjIndex; // hash -> bodies[offset]
-	typedef BodyIndex::iterator             IndexIter;
+	typedef std::multimap<uint32, uint32>   IndexMap; // hash -> vec[offset]
+	typedef IndexMap::iterator              IndexIter;
 	typedef std::pair<IndexIter, IndexIter> IndexRange;
 	typedef PodVector<uint8>::type          SccMap;
 	struct MinimizeRule;
@@ -468,16 +468,18 @@ private:
 	bool addConstraints();
 	// ------------------------------------------------------------------------
 	void writeBody(const BodyInfo& body, std::ostream&) const;
-	void transform(const PrgBody& body, BodyInfo& out)  const;
+	bool transform(const PrgBody& body, BodyInfo& out)  const;
 	void transform(const MinimizeRule&, BodyInfo& out)  const;
-	Var  getFalseAtom() const;
+	Var      getFalseId()      const { return 0; }
+	PrgAtom* getFalseAtom()    const { return atoms_[0]; }
+	Var      findLpFalseAtom() const;
 	Rule           rule_;        // active rule
 	BodyInfo       activeBody_;  // simplified body of active rule
 	VarVec         activeHead_;  // simplified head of active rule
 	RuleState      ruleState_;   // which atoms appear in the active rule?
 	RuleList       extended_;    // extended rules to be translated
-	BodyIndex      bodyIndex_;   // hash -> body
-	DisjIndex      disjIndex_;   // hash -> disjunction
+	IndexMap       bodyIndex_;   // hash -> body
+	IndexMap       disjIndex_;   // hash -> disjunction
 	BodyList       bodies_;      // all bodies
 	AtomList       atoms_;       // all atoms
 	AtomList       sccAtoms_;    // atoms that are strongly connected
