@@ -26,3 +26,6 @@ void printModel(const Clasp::SymbolTable& sym, const Clasp::Model& model);
 
 void example1();
 void example2();
+#if WITH_THREADS
+void example3();
+#endif
