@@ -86,139 +86,198 @@ void ClaspConfig::prepare(SharedContext& ctx) {
 	BasicSatConfig::prepare(ctx);
 }
 /////////////////////////////////////////////////////////////////////////////////////////
-// ClaspFacade::SolveImpl/AsyncResult
+// ClaspFacade::SolveImpl/SolveStrategy/AsyncResult
 /////////////////////////////////////////////////////////////////////////////////////////
-static const ClaspFacade::Result result_unknown = {uint8(0),uint8(0)};
-struct ClaspFacade::SolveImpl {
-	typedef SingleOwnerPtr<SolveAlgorithm> AlgoPtr;
-	enum State { state_done = 0, state_running = 1 };
-	SolveImpl() : algo(0) { state = state_done; signal = 0; result = result_unknown; }
-	~SolveImpl() { join(); }
-	AlgoPtr            algo;
+struct ClaspFacade::SolveStrategy {
+public:
+	static const int SIGCANCEL = 9;
+	enum State { state_start = 0, state_running = 1, state_result = 2, state_model = 3, state_done = 6 };
+	SolveStrategy()         { state = signal = 0; algo = 0; handler = 0; }
+	virtual ~SolveStrategy(){}
+	bool running() const    { return (state & state_running) != 0; }
+	bool interrupt(int sig) {
+		if (!running()) { return false; }
+		if (!signal || sig < signal) { signal = sig; } 
+		return cancel(sig);
+	}
+	void solve(ClaspFacade& f, SolveAlgorithm* algo, EventHandler* h) {
+		this->algo    = algo;
+		this->handler = h;
+		state = signal= 0;
+		doSolve(f);
+	}
+	virtual void release()  {}
+	virtual bool cancel(int){ return algo->interrupt(); }
 	Clasp::atomic<int> state;
 	Clasp::atomic<int> signal;
-	Result             result;
-	bool ready()  const    { return state != state_running; }
-	bool running()const    { return state == state_running; }
-	void reset()           { state = signal = 0; result = result_unknown; algo->resetSolve(); }
-	bool interrupt(int sig){
-		if (running() && algo->interrupt()) {
-			if (!signal || sig < signal) { signal = sig; }
-			return true;
+	SolveAlgorithm*    algo;
+	EventHandler*      handler;
+protected:
+	void solveImpl(ClaspFacade& f, State end);
+	virtual void doSolve(ClaspFacade& f) = 0;
+};
+struct ClaspFacade::SolveImpl {
+	typedef SingleOwnerPtr<SolveAlgorithm> AlgoPtr;
+	typedef SingleOwnerPtr<Enumerator>     EnumPtr;
+	SolveImpl() : en(0), algo(0), active(0) { }
+	~SolveImpl() { reset(); }
+	void init(SolveAlgorithm* algo, Enumerator* en) {
+		this->en   = en;
+		this->algo = algo;
+		this->algo->setEnumerator(*en);
+	}
+	void reset() {
+		if (active)    { interrupt(SolveStrategy::SIGCANCEL); active->release(); active = 0; }
+		if (algo.get()){ algo->resetSolve(); }
+		if (en.get())  { en->reset(); }
+	}
+	void prepare(SharedContext& ctx, SharedMinimizeData* min, int numM) {
+		CLASP_FAIL_IF(active, "Solve operation still active");
+		int limit = en->init(ctx, min, numM);
+		algo->setEnumLimit(limit ? static_cast<uint64>(limit) : UINT64_MAX);
+	}
+	bool update(const Solver& s, const Model& m) { return !active->handler || active->handler->onModel(s, m); }
+	bool interrupt(int sig)                      { return active && active->interrupt(sig); }
+	bool                      solving()   const  { return active && active->running();   }
+	const Model*              lastModel() const  { return en.get() ? &en->lastModel() : 0; }
+	const SharedMinimizeData* minimizer() const  { return en.get() ? en->minimizer() : 0; } 
+	bool                      optimize()  const  { return en.get() && en->optimize(); }
+	int                       modelType() const  { return en.get() ? en->modelType() : 0; }
+	EnumPtr        en;
+	AlgoPtr        algo;
+	SolveStrategy* active;
+};
+void ClaspFacade::SolveStrategy::solveImpl(ClaspFacade& f, State done) {
+	if (state != state_running){ state = state_running; }
+	if (f.result().flags)      { state = done; signal = f.result().signal; return; }
+	struct OnExit {
+		OnExit(SolveStrategy* s, ClaspFacade* x, int st) : self(s), facade(x), endState(st), more(true) {}
+		~OnExit() { 
+			facade->stopStep(self->signal, !more);
+			if (self->handler) { self->handler->onEvent(StepReady(facade->summary())); }
+			self->state  = endState;
 		}
-		return false;
-	}
-	void solve(ClaspFacade& f, EventHandler* h, bool async) {
-		assert(f.config_);
-		CLASP_ASSERT_CONTRACT_MSG(ready(), "Concurrent solve operations not supported!");
-		join(); // make sure that worker thread is ready
-		if (!f.ok() || f.result().unsat()) { return; }
-		if (h != f.ctx.eventHandler()) { f.stepHandler_ = h; }
-		if (async) { algo->enableInterrupts(); }
-		state      = SolveImpl::state_running;
-		if (!async){ solveImpl(f, state_done);}
-		else       { aSolve(f); }
-	}
-	void solveImpl(ClaspFacade& f, State done) {
-		struct OnExit {
-			OnExit(SolveImpl* s, ClaspFacade* x, int st) : self(s), facade(x), endState(st), more(true) {}
-			~OnExit() { 
-				self->result = facade->stopStep(self->signal, !more);
-				self->state  = endState;
-			}
-			SolveImpl*    self;
-			ClaspFacade*  facade;
-			int           endState;
-			bool          more;
-		} scope(this, &f, done);
-		f.step_.solveTime = f.step_.unsatTime = RealTime::getTime();
-		scope.more = algo->solve(f.ctx, f.assume_, &f);
-	}
+		SolveStrategy* self;
+		ClaspFacade*   facade;
+		int            endState;
+		bool           more;
+	} scope(this, &f, done);
+	f.step_.solveTime = f.step_.unsatTime = RealTime::getTime();
+	scope.more = algo->solve(f.ctx, f.assume_, &f);
+}
+
 #if WITH_THREADS
-	static const int SIGASYNC = 32;
-	void wait()            {
-		for (unique_lock<Clasp::mutex> lock(mutex); !ready();) { condition.wait(lock); }
-	}
-	bool waitFor(double s) {
-		for (unique_lock<Clasp::mutex> lock(mutex); !ready();) {
-			try { condition.wait_for(lock, tbb::tick_count::interval_t(s)); break; }
-			catch (const std::runtime_error&) {
-				if (s < 0) { throw; }
-				// Due to a bug in the computation of the wait time in some tbb versions 
-				// for linux wait_for might fail with eid_condvar_wait_failed.
-				// See: http://software.intel.com/en-us/forums/topic/280012
-				// Ignore the error and retry the wait - the computed wait time will be valid, eventually.
-			}
-		}
-		return ready();
-	}
-	void join() {
-		if (task.joinable()) { task.join(); }
-	}
-	void aSolve(ClaspFacade& f) {
+struct ClaspFacade::AsyncSolve : public SolveStrategy, public EventHandler {
+	static EventHandler* asyncModelHandler() { return reinterpret_cast<EventHandler*>(0x1); }
+	AsyncSolve() { refs = 1; }
+	virtual void doSolve(ClaspFacade& f) {
+		if (handler == AsyncSolve::asyncModelHandler()) { handler = this; }
+		algo->enableInterrupts();
+		state  = state_running;
+		result = f.result();
+		if (result.flags) { signal = result.signal; state = state_done; return; }
 		// start solve in worker thread
-		Clasp::thread(std::mem_fun(&SolveImpl::aSolveImpl), this, &f).swap(task);
+		Clasp::thread(std::mem_fun(&AsyncSolve::threadMain), this, &f).swap(task);
 	}
-	void aSolveImpl(ClaspFacade* f) {
+	virtual bool cancel(int sig) {
+		return SolveStrategy::cancel(sig) && (sig != SIGCANCEL || wait());
+	}
+	void threadMain(ClaspFacade* f) {
 		try         { solveImpl(*f, state_running); }
 		catch (...) { result.flags |= Result::EXT_ERROR; }
 		{
-			unique_lock<Clasp::mutex> lock(this->mutex);
-			state = state_done;
+			unique_lock<Clasp::mutex> lock(this->mqMutex);
+			result = f->result();
+			state  = state_done;
 		}
-		condition.notify_one();
+		mqCond.notify_one();
 	}
-	Clasp::thread             task;
-	Clasp::mutex              mutex;
-	Clasp::condition_variable condition;
-#else
-	void join() {}
-	void aSolve(ClaspFacade& f) { solveImpl(f, state_done); }
-#endif
+	virtual void release() {
+		if      (--refs == 1) { interrupt(SIGCANCEL); }
+		else if (refs   == 0) { join(); delete this; }
+	}
+	bool hasResult()const { return (state & state_result)  != 0; }
+	bool next() {
+		if (state != state_model) { return false; }
+		unique_lock<Clasp::mutex> lock(mqMutex);
+		state = state_running;
+		mqCond.notify_one();
+		return true;
+	}
+	bool wait(double s = -1.0) {
+		if (state == state_start) { return false; }
+		if (signal != 0)          { next(); }
+		for (unique_lock<Clasp::mutex> lock(mqMutex); !hasResult(); ) {
+			if (s < 0.0) { mqCond.wait(lock); }
+			else {
+				try { 
+					mqCond.wait_for(lock, tbb::tick_count::interval_t(s));
+					if (!hasResult()) { return false; }
+				}
+				catch (const std::runtime_error&) {
+					// Due to a bug in the computation of the wait time in some tbb versions 
+					// for linux wait_for might fail with eid_condvar_wait_failed.
+					// See: http://software.intel.com/en-us/forums/topic/280012
+					// Ignore the error and retry the wait - the computed wait time will be valid, eventually.
+				}
+			}
+		}
+		if (state == state_done) { join(); }
+		return true;
+	}
+	bool onModel(const Solver&, const Model&) {
+		const Result sat = {Result::SAT, 0};
+		unique_lock<Clasp::mutex> lock(mqMutex);
+		state = state_model;
+		result= sat;
+		mqCond.notify_one();
+		while (state == state_model && signal == 0) { mqCond.wait(lock); }
+		return signal == 0;
+	}
+	void join() { if (task.joinable()) { task.join(); } }
+	Clasp::thread             task;   // async solving thread
+	Clasp::mutex              mqMutex;// protects mqCond
+	Clasp::condition_variable mqCond; // for iterating over models one by one
+	Clasp::atomic<int>        refs;   // 1 + #AsyncResult objects
+	Result                    result; // result of async operation
 };
-
-#if WITH_THREADS
-ClaspFacade::AsyncResult::AsyncResult(SolveImpl& x) : state_(&x) {}
+ClaspFacade::AsyncResult::AsyncResult(SolveImpl& x) : state_(static_cast<AsyncSolve*>(x.active)) { ++state_->refs; }
+ClaspFacade::AsyncResult::~AsyncResult() { state_->release(); }
+ClaspFacade::AsyncResult::AsyncResult(const AsyncResult& x) : state_(x.state_) { ++state_->refs; }
 int  ClaspFacade::AsyncResult::interrupted()        const { return state_->signal; }
 bool ClaspFacade::AsyncResult::error()              const { return ready() && state_->result.error(); }
-bool ClaspFacade::AsyncResult::ready()              const { return state_->ready(); }
+bool ClaspFacade::AsyncResult::ready()              const { return state_->hasResult(); }
 bool ClaspFacade::AsyncResult::ready(Result& r)     const { if (ready()) { r = get(); return true; } return false; }
+bool ClaspFacade::AsyncResult::running()            const { return state_->running(); }
+void ClaspFacade::AsyncResult::wait()               const { state_->wait(-1.0); }
+bool ClaspFacade::AsyncResult::waitFor(double sec)  const { return state_->wait(sec); }
+void ClaspFacade::AsyncResult::next()               const { state_->next(); }
+bool ClaspFacade::AsyncResult::cancel()             const { return state_->interrupt(AsyncSolve::SIGCANCEL); }
 ClaspFacade::Result ClaspFacade::AsyncResult::get() const {
-	wait();
-	if (state_->result.error()) { throw std::runtime_error("Async operation failed!"); }
-	return state_->result;
-}
-void ClaspFacade::AsyncResult::wait()  const { 
 	state_->wait();
-	state_->join();
+	if (!state_->result.error()) { return state_->result; }
+	throw std::runtime_error("Async operation failed!");
 }
-bool ClaspFacade::AsyncResult::waitFor(double sec) const {
-	if (state_->waitFor(sec)) {
-		state_->join();
-		return true;
-	}
-	return false;
+bool ClaspFacade::AsyncResult::end() const { 
+	// first running() handles case where iterator is outdated (state was reset to start)
+	// second running() is used to distinguish models from final result (summary)
+	return !running() || !get().sat() || !running();
 }
-bool ClaspFacade::AsyncResult::cancel()const { 
-	if (state_->interrupt(SolveImpl::SIGASYNC)) {
-		wait();
-		return true;
-	}
-	return false;
+const Model& ClaspFacade::AsyncResult::model() const { 
+	CLASP_FAIL_IF(state_->state != AsyncSolve::state_model, "Invalid iterator access!");
+	return const_cast<const SolveAlgorithm*>(state_->algo)->enumerator()->lastModel();
 }
 #endif
 /////////////////////////////////////////////////////////////////////////////////////////
 // ClaspFacade
 /////////////////////////////////////////////////////////////////////////////////////////
-ClaspFacade::ClaspFacade() : config_(0) {
-}
+ClaspFacade::ClaspFacade() : config_(0) {}
 ClaspFacade::~ClaspFacade() { }
 void ClaspFacade::discardProblem() {
 	config_  = 0;
 	builder_ = 0;
-	enum_    = 0;
 	lpStats_ = 0;
-	solve_   = new SolveImpl();
+	solve_   = 0;
 	accu_    = 0;
 	step_.init(*this);
 	if (ctx.numConstraints() || ctx.numVars()) { ctx.reset(); }
@@ -227,16 +286,16 @@ void ClaspFacade::init(ClaspConfig& config, bool discard) {
 	if (discard) { discardProblem(); }
 	ctx.setConfiguration(0, false); // force reload of configuration once done
 	config_ = &config;
-	if ((enum_ = config.enumerate.createEnumerator()).get() == 0) {
-		enum_ = EnumOptions::nullEnumerator();
-	}
-	if (config.solve.numSolver() > 1 && !enum_->supportsParallel()) {
+	SolveImpl::EnumPtr e(config.enumerate.createEnumerator());
+	if (e.get() == 0) { e = EnumOptions::nullEnumerator(); }
+	if (config.solve.numSolver() > 1 && !e->supportsParallel()) {
 		ctx.report(warning(Event::subsystem_facade, "Selected reasoning mode implies #Threads=1."));
 		config.setSolvers(1);
 	}
 	ctx.setConfiguration(&config, false); // prepare and apply config
-	solve_->algo = config.solve.createSolveObject();
-	solve_->algo->setEnumerator(*enum_);
+	if (!solve_.get()) { solve_ = new SolveImpl(); }
+	SolveImpl::AlgoPtr a(config.solve.createSolveObject());
+	solve_->init(a.release(), e.release());
 	if (discard) { startStep(0); }
 }
 
@@ -284,7 +343,7 @@ Asp::LogicProgram& ClaspFacade::startAsp(ClaspConfig& config, bool allowUpdate)
 	return *p;
 }
 ProgramBuilder& ClaspFacade::update(bool reloadConfig) {
-	CLASP_ASSERT_CONTRACT(builder_.get() && solve_->ready());
+	CLASP_ASSERT_CONTRACT(builder_.get() && !solving());
 	CLASP_ASSERT_CONTRACT_MSG(step_.result.signal != SIGINT, "Interrupt not handled!");
 	solve_->reset();
 	if (reloadConfig) { init(*config_, false); }
@@ -305,7 +364,7 @@ bool ClaspFacade::terminate(int signal) {
 }
 
 bool ClaspFacade::prepare(EnumMode enumMode) {
-	CLASP_ASSERT_CONTRACT(config_ && solve_->ready());
+	CLASP_ASSERT_CONTRACT(config_ && !solving());
 	bool ok = this->ok();
 	SharedMinimizeData* m = 0;
 	EnumOptions& en       = config_->enumerate;
@@ -319,14 +378,9 @@ bool ClaspFacade::prepare(EnumMode enumMode) {
 		}
 	}
 	if (ok && (!ctx.frozen() || (ok = ctx.unfreeze()) == true)) {
-		if (enumMode == enum_static) {
-			// Step literal not needed in this step.
-			ok = ctx.addUnary(ctx.stepLiteral());
-		}
-		if (ok) {
-			int limit = enum_->init(ctx, m && m != enum_->minimizer() ? m->share() : m, en.numModels);
-			solve_->algo->setEnumLimit(limit ? static_cast<uint64>(limit) : UINT64_MAX);
-		}
+		// Step literal only needed for volatile enumeration
+		ok = (enumMode == enum_volatile || ctx.addUnary(ctx.stepLiteral()));
+		if (ok) { solve_->prepare(ctx, m ? m->share() : 0, en.numModels); }
 	}
 	if (!accu_.get()) { builder_ = 0; }
 	return (ok && ctx.endInit()) || (stopStep(0, true), false);
@@ -337,29 +391,39 @@ void ClaspFacade::assume(Literal p) {
 void ClaspFacade::assume(const LitVec& ext) {
 	assume_.insert(assume_.end(), ext.begin(), ext.end());
 }
-bool ClaspFacade::solving() const { return solve_->running(); }
+bool ClaspFacade::solving() const { return solve_.get() && solve_->solving(); }
 
 const ClaspFacade::Summary& ClaspFacade::shutdown() {
 	if      (!config_) { step_.init(*this); }
-	else if (solving()){ terminate(SIGINT); solve_->join(); }
+	else if (solving()){ terminate(SIGINT); }
 	else               { stopStep(0, false); }
 	return accu_.get() && accu_->step ? *accu_ : step_;
 }
 
 ClaspFacade::Result ClaspFacade::solve(EventHandler* handler) {
-	solve_->solve(*this, handler, false);
+	CLASP_ASSERT_CONTRACT(!solving());
+	struct SyncSolve : public SolveStrategy {
+		SyncSolve(SolveImpl& s) : x(&s) { x->active = this; }
+		~SyncSolve()                    { x->active = 0;    }
+		virtual void doSolve(ClaspFacade& f) { solveImpl(f, state_done); }
+		SolveImpl* x;
+	} syncSolve(*solve_);
+	syncSolve.solve(*this, solve_->algo.get(), handler);
 	return result();
 }
-
 #if WITH_THREADS
 ClaspFacade::AsyncResult ClaspFacade::solveAsync(EventHandler* handler) {
-	solve_->solve(*this, handler, true);
+	CLASP_ASSERT_CONTRACT(!solving());
+	solve_->active = new AsyncSolve();
+	solve_->active->solve(*this, solve_->algo.get(), handler);
 	return AsyncResult(*solve_);
 }
+ClaspFacade::AsyncResult ClaspFacade::startSolveAsync() {
+	return solveAsync(AsyncSolve::asyncModelHandler());
+}
 #endif
 
 void ClaspFacade::startStep(uint32 n) {
-	stepHandler_    = 0;
 	step_.init(*this);
 	step_.totalTime = -RealTime::getTime();
 	step_.cpuTime   = -ProcessTime::getTime();
@@ -381,7 +445,6 @@ ClaspFacade::Result ClaspFacade::stopStep(int signal, bool complete) {
 		if (signal)   { res.flags|= uint8(Result::EXT_INTERRUPT); }
 		step_.result = res;
 		accuStep();
-		if (stepHandler_) { stepHandler_->onEvent(StepReady(step_)); }
 		ctx.report(StepReady(step_)); 
 	}
 	return result();
@@ -403,7 +466,7 @@ void ClaspFacade::accuStep() {
 bool ClaspFacade::onModel(const Solver& s, const Model& m) {
 	step_.unsatTime = RealTime::getTime();
 	if (++step_.numEnum == 1) { step_.satTime = step_.unsatTime - step_.solveTime; }
-	return !stepHandler_ || stepHandler_->onModel(s, m);
+	return solve_->update(s, m);
 }
 ExpectedQuantity::ExpectedQuantity(double d) : rep(d >= 0 ? d : -std::min(-d, 3.0)) {}
 ExpectedQuantity::Error ExpectedQuantity::error() const {
@@ -521,15 +584,15 @@ ExpectedQuantity ClaspFacade::getStat(const SharedContext& ctx, const char* key,
 /////////////////////////////////////////////////////////////////////////////////////////
 void ClaspFacade::Summary::init(ClaspFacade& f)  { std::memset(this, 0, sizeof(Summary)); facade = &f;}
 int ClaspFacade::Summary::stats()          const { return ctx().master()->stats.level(); }
-const Model* ClaspFacade::Summary::model() const { return facade->enum_.get() ? &facade->enum_->lastModel() : 0; }
-const SharedMinimizeData* ClaspFacade::Summary::costs()const { return facade->enum_.get() ? facade->enum_->minimizer() : 0;  }
+const Model* ClaspFacade::Summary::model() const { return facade->solve_.get() ? facade->solve_->lastModel() : 0; }
+const SharedMinimizeData* ClaspFacade::Summary::costs()const { return facade->solve_.get() ? facade->solve_->minimizer() : 0;  }
 const char* ClaspFacade::Summary::consequences() const {
-	int mt = facade->enum_.get() ? facade->enum_->modelType() : 0;
+	int mt = facade->solve_.get() ? facade->solve_->modelType() : 0;
 	if ( (mt & CBConsequences::brave_consequences)    == CBConsequences::brave_consequences)   { return "Brave"; }
 	if ( (mt & CBConsequences::cautious_consequences) == CBConsequences::cautious_consequences){ return "Cautious"; }
 	return 0;
 }
-bool ClaspFacade::Summary::optimize()  const { return (facade->enum_.get() && facade->enum_->optimize()) || (model() && model()->opt); }
+bool ClaspFacade::Summary::optimize()  const { return (facade->solve_.get() && facade->solve_->optimize()) || (model() && model()->opt); }
 bool ClaspFacade::Summary::optimum()   const { return (model() && model()->opt) || (costs() && complete()); }
 uint64 ClaspFacade::Summary::optimal() const { 
 	const Model* m = model();
