@@ -58,6 +58,7 @@ typedef std::pair<uint32, uint32> UPair;
 #define ARG(a) ->a
 #define VALUE (_val_)
 #define TEMP  (_tmp_)
+#define HEU_PARAM "hparam"
 /////////////////////////////////////////////////////////////////////////////////////////
 // Functions for parsing/converting certain clasp types
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -156,7 +157,13 @@ struct ClaspCliConfig::ParseContext : public ProgramOptions::ParseContext {
 	typedef ProgramOptions::SharedOptPtr   OptPtr;
 	typedef OptionContext::option_iterator OptionIter;
 	ParseContext() : root(0) {}
-	void init() { first = root->find("configuration"); }
+	void init() { 
+		first = root->find("configuration"); 
+		OptionIter it = root->find(HEU_PARAM);
+		root->addAlias("vsids-decay", it);
+		root->addAlias("berk-max", it);
+		root->addAlias("vmtf-mtf", it);
+	}
 	
 	OptPtr getOption(const char* name, FindType ft) {
 		OptionIter it  = (root->begin() + root->findImpl(name, ft, 3u, frame->config).first->second);
@@ -291,7 +298,14 @@ ClaspCliConfig::ScopedSet::ScopedSet(ClaspCliConfig& s, uint8 mode, uint32 sId)
 	s.cliMode = mode;
 }
 ClaspCliConfig::ScopedSet::~ScopedSet() { self->cliId = self->cliMode = 0; }
-
+ClaspCliConfig::RawConfig::RawConfig(const char* name) {
+	raw.append(1, '/').append(name ? name : "").append("\0/\0", 3);	
+}
+void ClaspCliConfig::RawConfig::addArg(const char* arg) {
+	*raw.rbegin() = ' ';
+	raw.append(arg ? arg : "").append(1, '\0');
+}
+void ClaspCliConfig::RawConfig::addArg(const std::string& arg) { addArg(arg.c_str()); }
 ClaspCliConfig::ClaspCliConfig()  {}
 ClaspCliConfig::~ClaspCliConfig() {}
 ClaspCliConfig::ProgOption* ClaspCliConfig::createOption(int o) {  return new ProgOption(*this, o); }
@@ -442,7 +456,7 @@ bool ClaspCliConfig::setDefaults(UserConfig* active, uint32 sId, const ParsedOpt
 }
 
 bool ClaspCliConfig::set(const ConfigIter& config, bool allowConfig, const ParsedOpts& exclude, ParsedOpts* out) {
-	if (!opts_.get()) { init(new OptionContext(), true); }
+	if (!opts_.get() || !opts_->root.get()) { init(new OptionContext(), true); }
 	ParseContext::Frame frame = { {uint64(0),uint64(0)}, config.name(), &exclude, out, (allowConfig ? -2 : 0) };
 	opts_->frame = &frame;
 	ProgramOptions::parseCommandString(config.args(), *opts_, ProgramOptions::command_line_allow_flag_value);
@@ -497,11 +511,11 @@ const ClaspCliConfig::ParsedOpts& ClaspCliConfig::finalizeParsed(UserConfig* act
 		exclude.add("del-max");
 		exclude.add("del-grow");
 	}
-	if (parsed.count("heuristic")) {
+	CLASP_FAIL_IF(parsed.count(HEU_PARAM) && !parsed.count("heuristic"), "'--" HEU_PARAM "' requires '--heuristic'");
+	if (parsed.count("heuristic") && !parsed.count(HEU_PARAM)) {
 		if (!copied) { exclude = parsed; copied = true; }
-		if (active->solver(0).heuId != Heuristic_t::heu_vsids)  { exclude.add("vsids-decay"); }
-		if (active->solver(0).heuId != Heuristic_t::heu_vmtf)   { exclude.add("vmtf-mtf"); }
-		if (active->solver(0).heuId != Heuristic_t::heu_berkmin){ exclude.add("berk-max"); }
+		exclude.add(HEU_PARAM);
+		active->addSolver(0).heuParam = 0;
 	}
 	return !copied ? parsed : exclude;
 }
@@ -566,12 +580,18 @@ void ClaspCliConfig::error(int opt) const {
 #define CLASP_SEARCH_RESTART_OPTIONS
 #define CLASP_SEARCH_REDUCE_OPTIONS
 #include <clasp/cli/clasp_cli_options.inl>
-	}	
+	}
 	const char* end = optName;
 	while (*end && *end != ',' && *end != '!') { ++end; }
 	throw ProgramOptions::UnknownOption(isGenerator() ? "<clasp>" : "<tester>", std::string(optName, end - optName));
 }
 
+bool ClaspCliConfig::setConfig(const RawConfig& config, ProblemType t) {
+	ProgramOptions::ParsedOptions exclude;
+	reset();
+	return set(config.iterator(), true, exclude, &exclude) && opts_->root->assignDefaults(exclude) && finalize(exclude, t, true);
+}
+
 void validate(const char* ctx, const SolverParams& solver, const SolveParams& search) {
 	if (!ctx) { ctx = "<clasp>"; }
 	const ReduceParams& reduce = search.reduce;
@@ -584,4 +604,5 @@ void validate(const char* ctx, const SolverParams& solver, const SolveParams& se
 	CLASP_FAIL_IF(hasSched  && reduce.fReduce() == 0.0f && !reduce.growSched.defaulted(), "'%s': 'no-deletion': deletion strategies disabled!", ctx);
 	CLASP_FAIL_IF(!hasSched && reduce.fReduce() != 0.0f && !reduce.growSched.defaulted(), "'%s': 'deletion': deletion strategy required!", ctx);
 }
+
 }}
