@@ -20,6 +20,7 @@
 #include <clasp/heuristics.h>
 #include <clasp/dependency_graph.h>
 #include <clasp/enumerator.h>
+#include <clasp/clause.h>
 #include <algorithm>
 #include <limits>
 #include <cstdlib>
@@ -616,6 +617,36 @@ template class ClaspVsids_t<VsidsScore>;
 /////////////////////////////////////////////////////////////////////////////////////////
 // DomainHeuristic selection strategy
 /////////////////////////////////////////////////////////////////////////////////////////
+class DomainHeuristic::DomMinimize : public MinimizeConstraint {
+public:
+	explicit DomMinimize(const LitVec& lits) : MinimizeConstraint(createDataFrom(lits)){ }
+	// constraint interface
+	PropResult  propagate(Solver&, Literal, uint32&) { return PropResult(true, true); }
+	void        reason(Solver&, Literal, LitVec&)    { }
+	bool        simplify(Solver& s, bool)            { simplifyDB(s, nogoods_, false); return false; }
+	Constraint* cloneAttach(Solver&)                 { return 0; }
+	void        destroy(Solver*, bool);
+	// base interface
+	bool        attach(Solver&)                      { return true; }
+	bool        relax(Solver&, bool)                 { return true; }
+	bool        handleUnsat(Solver&, bool, LitVec&)  { return false;}
+	bool        integrate(Solver& s);
+	bool        handleModel(Solver& s);
+private:
+	static SharedMinimizeData* createDataFrom(const LitVec& lits) {
+		SumVec adjust(1, 0);
+		SharedMinimizeData* min = new (::operator new(sizeof(SharedMinimizeData) + ((1+lits.size())*sizeof(WeightLiteral)))) SharedMinimizeData(adjust, MinimizeMode_t::enumerate);
+		WeightLiteral* x = min->lits;
+		for (LitVec::const_iterator it = lits.begin(), end = lits.end(); it != end; ++it) {
+			*x++ = WeightLiteral(*it, 1);
+		}
+		*x++ = WeightLiteral(posLit(0), 1);
+		return min;
+	}
+	typedef PodVector<Constraint*>::type ConstraintDB;
+	ConstraintDB nogoods_;
+	LitVec       clause_;
+};
 struct DomainHeuristic::CmpSymbol {
 	bool operator()(const SymbolType& lhs, const SymbolType& rhs) const { return std::strcmp(lhs.name.c_str(), rhs.name.c_str()) < 0; }
 	bool operator()(const SymbolType& lhs, const char* rhs)       const { return std::strcmp(lhs.name.c_str(), rhs) < 0; }
@@ -667,7 +698,7 @@ bool DomainHeuristic::DomEntry::parse(const char*& key) {
 	prio = (uint16)Range<int>(0, INT16_MAX).clamp(std::abs(temp));
 	return true;
 }
-DomainHeuristic::DomainHeuristic(double d, const HeuParams& params) : ClaspVsids_t(d, params), solver_(0) {
+DomainHeuristic::DomainHeuristic(double d, const HeuParams& params) : ClaspVsids_t<DomScore>(d, params), solver_(0) {
 	frames_.push_back(Frame(0,UINT32_MAX));
 }
 DomainHeuristic::~DomainHeuristic() {
@@ -710,6 +741,11 @@ void DomainHeuristic::initScores(Solver& s, bool moms) {
 	DomEntry    last(&noSym, DomEntry::mod_init);
 	Literal     dyn;
 	DomPrio     prio;
+	LitVec      domMin;
+	EnumerationConstraint*    c = static_cast<EnumerationConstraint*>(s.enumerationConstraint());
+	const MinimizeConstraint* m = c ? c->minimizer() : 0;
+	const uint32 gPref          = s.configuration().domPref;
+	minLits_                    = s.configuration().domMod >= uint32(gmod_max_value) && !m ? &domMin : 0;
 	for (SymbolMap::iterator it = domRange.first; it != domRange.second; ++it) {
 		const char* key = it->name.c_str();
 		if (!matchDom(key, name)) { continue; }
@@ -748,11 +784,13 @@ void DomainHeuristic::initScores(Solver& s, bool moms) {
 	}
 	endInit(last, prio);
 	// apply global preferences
-	if (uint32 gPref = s.configuration().domPref) {
-		uint32 gModf = Range32(gmod_level, gmod_max_value).clamp(s.configuration().domMod);
+	if (gPref) {
+		uint32 gModf = s.configuration().domMod;
 		uint32 graph = gpref_scc | gpref_hcc | gpref_disj;
-		const MinimizeConstraint* m = 0;
-		if ((gPref & gpref_min) != 0 && s.enumerationConstraint() && (m = static_cast<EnumerationConstraint*>(s.enumerationConstraint())->minimizer()) != 0) {
+		if (gModf >= uint32(gmod_max_value)) {
+			gModf = gModf == 6 ? gmod_false : gmod_true;
+		}
+		if ((gPref & gpref_min) != 0 && m) {
 			weight_t w = -1; 
 			int16    x = gpref_show;
 			for (const WeightLiteral* it = m->shared()->lits; !isSentinel(it->first); ++it) {
@@ -786,6 +824,18 @@ void DomainHeuristic::initScores(Solver& s, bool moms) {
 			}
 		}
 	}
+	LitVec::iterator j = domMin.begin();
+	for (LitVec::const_iterator it = domMin.begin(), end = domMin.end(); it != end; ++it) {
+		if (s.value(it->var()) == value_free && score_[it->var()].level > 0 && !s.seen(*it)) {
+			s.markSeen(*it);
+			*j++ = *it;
+		}
+	}
+	domMin.erase(j, domMin.end());
+	for (LitVec::const_iterator it = domMin.begin(), end = domMin.end(); it != end; ++it) { s.clearSeen(it->var()); }
+	if (!domMin.empty()) {
+		c->setMinimizer(new DomMinimize(domMin));
+	}
 }
 void DomainHeuristic::addAction(Solver& s, const SymbolType& pre, const DomEntry& e) {
 	assert(e.mod < DomEntry::mod_init);
@@ -793,7 +843,10 @@ void DomainHeuristic::addAction(Solver& s, const SymbolType& pre, const DomEntry
 	if (s.isTrue(pre.lit)) {
 		if      (e.mod == DomEntry::mod_level) { score_[domVar].level = e.val; }
 		else if (e.mod == DomEntry::mod_factor){ score_[domVar].factor= e.val; }
-		else if (e.mod == DomEntry::mod_sign)  { s.setPref(domVar, ValueSet::user_value, e.signValue()); }
+		else if (e.mod == DomEntry::mod_sign)  { 
+			s.setPref(domVar, ValueSet::user_value, e.signValue());
+			if (minLits_) { minLits_->push_back(e.signValue() == falseValue(e.sym->lit) ? e.sym->lit : ~e.sym->lit); }
+		}
 	}
 	else {
 		if (score_[domVar].domKey == dPrios_.size()) { dPrios_.push_back(DomPrio()); }
@@ -806,11 +859,14 @@ void DomainHeuristic::addAction(Solver& s, Literal x, uint32 m, int16 lev) {
 		if ((m & gmod_level) != 0) {
 			score_[x.var()].level = lev;
 		}
-		if ((m & (gmod_spos|gmod_sneg)) != 0 && !s.pref(x.var()).has(ValueSet::user_value)) {
-			s.setPref(x.var(), ValueSet::user_value, (m & gmod_spos) != 0 ? trueValue(x) : falseValue(x));
-		}
 		score_[x.var()].setDom((uint32)dPrios_.size());
 	}
+	if (x.var() && (m & (gmod_spos|gmod_sneg)) != 0) {
+		if (!s.pref(x.var()).has(ValueSet::user_value)) {
+			s.setPref(x.var(), ValueSet::user_value, (m & gmod_spos) != 0 ? trueValue(x) : falseValue(x));
+		}
+		if (minLits_) { minLits_->push_back((m & gmod_spos) != 0 ? ~x : x); }
+	}	
 }
 
 Literal DomainHeuristic::doSelect(Solver& s) {
@@ -867,5 +923,30 @@ void DomainHeuristic::undoLevel(Solver& s) {
 		frames_.pop_back();
 	}
 }
+
+void DomainHeuristic::DomMinimize::destroy(Solver* s, bool b) {
+	while (!nogoods_.empty()) {
+		nogoods_.back()->destroy(s, b);
+		nogoods_.pop_back();
+	}
+	MinimizeConstraint::destroy(s, b);
+}
+bool DomainHeuristic::DomMinimize::handleModel(Solver& s) {
+	clause_.push_back(~s.sharedContext()->stepLiteral());
+	for (const WeightLiteral* lits = shared_->lits; !isSentinel(lits->first); ++lits) {
+		if (s.isTrue(lits->first)) { clause_.push_back(~lits->first); }
+	}
+	return true;
+}
+bool DomainHeuristic::DomMinimize::integrate(Solver& s) {
+	if (clause_.empty()) { return true; }
+	// at least one of the true literal must be false
+	ClauseInfo e(Constraint_t::learnt_other);
+	ClauseCreator::Result ret = ClauseCreator::create(s, clause_, ClauseCreator::clause_no_add, e);
+	clause_.clear();
+	if (ret.local) { nogoods_.push_back(ret.local); }
+	return ret.ok();
+}
+
 template class ClaspVsids_t<DomScore>;
 }
