@@ -164,8 +164,7 @@ void LogicProgram::dispose(bool force) {
 	else if (incData_) {
 		// clean up atoms
 		// reset prop queue
-		Var falseAtom = 0;
-		propQ_.assign(1, 0);
+		propQ_.assign(1, getFalseId());
 		uint32 startAux = incData_->startAux;
 		assert(startAux <= atoms_.size());
 		// remove rule associations
@@ -197,8 +196,7 @@ void LogicProgram::dispose(bool force) {
 				if (v != value_free) { assignValue(a, v); }
 			}
 			else if (!a->eq() && a->value() == value_false) {
-				if (!falseAtom) { a->resetId(falseAtom = i, false); propQ_.push_back(i); }
-				else            { a->setEq(falseAtom); }
+				a->setEq(getFalseId());
 			}
 		}
 		// delete any introduced aux atoms
@@ -219,6 +217,7 @@ bool LogicProgram::doStartProgram() {
 	// atom 0 is always false
 	atoms_.push_back( new PrgAtom(0, false) );
 	assignValue(getAtom(0), value_false);
+	getFalseAtom()->setLiteral(negLit(0));
 	nonHcfCfg_= 0;
 	incData_  = 0;
 	ctx()->symbolTable().clear();
@@ -320,8 +319,7 @@ void LogicProgram::write(std::ostream& os) {
 	// write all bodies together with their heads
 	for (BodyList::iterator it = bodies_.begin(); it != bodies_.end(); ++it) {
 		PrgBody* b = *it;
-		if (b->relevant() && (b->hasVar() || b->value() == value_false)) {
-			transform(*b, activeBody_);
+		if (b->relevant() && (b->hasVar() || b->value() == value_false) && transform(*b, activeBody_)) {
 			writeBody(activeBody_, body);
 			if (b->hasHeads() && b->value() != value_false) {
 				for (PrgBody::head_iterator it = b->heads_begin(); it != b->heads_end(); ++it) {
@@ -352,9 +350,8 @@ void LogicProgram::write(std::ostream& os) {
 			}
 			else if (b->value() == value_false) {
 				// write integrity constraint
-				if (falseAtom == 0) {
-					falseAtom = newAtom();
-					setCompute(falseAtom, false);
+				if (falseAtom == 0 && (falseAtom = findLpFalseAtom()) == 0) {
+					setCompute(falseAtom = newAtom(), false);
 				}
 				os << activeBody_.ruleType() << " " << falseAtom << " " << body.str() << "\n";
 			}
@@ -610,7 +607,6 @@ void LogicProgram::transformIntegrity(uint32 maxAux) {
 	}
 	if (!integrity.empty() && (integrity.size() == 1 || (atoms_.size()/double(bodies_.size()) > 0.5 && integrity.size() / double(bodies_.size()) < 0.01))) {
 		uint32 A      = static_cast<uint32>(atoms_.size());
-		Var falseAtom = 0;
 		// transform integrity constraints
 		for (BodyList::size_type i = 0; i != integrity.size(); ++i) {
 			PrgBody* b = integrity[i];
@@ -625,7 +621,7 @@ void LogicProgram::transformIntegrity(uint32 maxAux) {
 			extended_.push_back(r = new Rule());
 			r->setType(CONSTRAINTRULE);
 			r->setBound(b->bound());
-			r->addHead(falseAtom);
+			r->addHead(getFalseId());
 			for (uint32 g = 0; g != b->size(); ++g) {
 				r->addToBody(b->goal(g).var(), !b->goal(g).sign());
 			}
@@ -633,7 +629,7 @@ void LogicProgram::transformIntegrity(uint32 maxAux) {
 			transformExtended();
 			setFrozen(true);
 			// propagate integrity condition to new rules
-			propQ_.push_back(falseAtom);
+			propQ_.push_back(getFalseId());
 			propagate(true);
 			b->markRemoved();
 		}
@@ -923,7 +919,7 @@ bool LogicProgram::addConstraints() {
 	ctx()->symbolTable().endInit();
 	CLASP_ASSERT_CONTRACT(ctx()->symbolTable().curBegin() == ctx()->symbolTable().end() || startAtom() <= ctx()->symbolTable().curBegin()->first);
 	// handle initial conflict, if any
-	if (!ctx()->ok() || !ctx()->addUnary(atoms_[0]->trueLit())) {
+	if (!ctx()->ok() || !ctx()->addUnary(getFalseAtom()->trueLit())) {
 		return false;
 	}
 	// add bodies from this step
@@ -1031,9 +1027,9 @@ RuleType LogicProgram::simplifyBody(const Rule& r, BodyInfo& info) {
 				if (r.bodyIsSet()) { resType = ENDRULE; break; }
 				continue;
 			}
-			else if (a->value() != value_weak_true) {
+			else if (a->value() != value_weak_true && r.type() != OPTIMIZERULE) {
 				// literal is true - drop from rule
-				if (r.type() != OPTIMIZERULE && (bound -= w) <= 0) {
+				if ((bound -= w) <= 0) {
 					while (!sBody.empty()) { ruleState_.clear(sBody.back().first.var()); sBody.pop_back(); }
 					pos = hash = 0;
 					break;
@@ -1188,7 +1184,7 @@ PrgBody* LogicProgram::getBodyFor(BodyInfo& body, bool addDeps) {
 	// no corresponding body exists, create a new object
 	bodyId        = (uint32)bodies_.size();
 	PrgBody* b    = PrgBody::create(*this, bodyId, body, addDeps);
-	bodyIndex_.insert(BodyIndex::value_type(body.hash, bodyId));
+	bodyIndex_.insert(IndexMap::value_type(body.hash, bodyId));
 	bodies_.push_back(b);
 	if (b->isSupported()) {
 		initialSupp_.push_back(bodyId);
@@ -1278,7 +1274,7 @@ PrgDisj* LogicProgram::getDisjFor(const VarVec& heads, uint32 headHash) {
 			getAtom(*it)->addSupport(edge);
 		}
 		if (headHash) {
-			disjIndex_.insert(DisjIndex::value_type(headHash, d->id()));
+			disjIndex_.insert(IndexMap::value_type(headHash, d->id()));
 		}
 	}
 	return d;
@@ -1292,7 +1288,7 @@ uint32 LogicProgram::update(PrgBody* body, uint32 oldHash, uint32 newHash) {
 		if (eqId == varMax) {
 			// No equivalent body found. 
 			// Add new entry to index
-			bodyIndex_.insert(BodyIndex::value_type(newHash, id));
+			bodyIndex_.insert(IndexMap::value_type(newHash, id));
 		}
 		return eqId;
 	}
@@ -1357,7 +1353,7 @@ PrgBody* LogicProgram::mergeEqBodies(PrgBody* b, Var rootId, bool hashEq, bool a
 	return b;
 }
 
-uint32 LogicProgram::getFalseAtom() const {
+uint32 LogicProgram::findLpFalseAtom() const {
 	for (VarVec::size_type i = 1; i < atoms_.size(); ++i) {
 		if (!atoms_[i]->eq() && atoms_[i]->value() == value_false) {
 			return i;
@@ -1379,21 +1375,27 @@ VarVec& LogicProgram::getSupportedBodies(bool sorted) {
 	return initialSupp_;
 }
 
-void LogicProgram::transform(const PrgBody& body, BodyInfo& out) const {
+bool LogicProgram::transform(const PrgBody& body, BodyInfo& out) const {
 	out.reset();
 	out.lits.reserve(body.size());
 	uint32 p = 0, end = body.size();
 	while (p != end && !body.goal(p).sign()) { ++p; }
 	uint32 R[2][2] = { {p, end}, {0, p} };
+	weight_t sw = 0, st = 0;
 	for (uint32 range = 0; range != 2; ++range) {
 		for (uint32 x = R[range][0]; x != R[range][1]; ++x) {
 			WeightLiteral wl(body.goal(x), body.weight(x));
 			if (getAtom(wl.first.var())->hasVar()) {
+				sw += wl.second;
 				out.lits.push_back(wl);
 			}
+			else if (wl.first.sign()) {
+				st += wl.second;
+			}
 		}
 	}
-	out.init(body.type(), body.bound(), 0, p);
+	out.init(body.type(), std::max(body.bound() - st, weight_t(0)), 0, p);
+	return sw >= out.bound();
 }
 
 void LogicProgram::transform(const MinimizeRule& body, BodyInfo& out) const {
