@@ -361,8 +361,7 @@ void DefaultUnfoundedCheck::updateAssignment(Solver& s) {
 			// an atom in the head of a choice rule became false
 			// normally head false -> body false and hence the head has its source autmatically removed
 			// for choice rules we must force source removal explicity
-			const BodyNode& B = graph_->getBody(atoms_[index].watch());
-			if (atoms_[index].hasSource() && !s.isFalse(B.lit)) {
+			if (atoms_[index].hasSource() && !s.isFalse(graph_->getBody(atoms_[index].watch()).lit)) {
 				atoms_[index].markSourceInvalid();
 				graph_->getAtom(index).visitSuccessors(RemoveSource(this, true));
 				propagateSource();
