@@ -219,6 +219,10 @@ WeightConstraint* WeightConstraint::createImpl(Solver& s, Literal W, WeightLitsR
 	assert(m && (reinterpret_cast<uintp>(m) & 7u) == 0);
 	SharedContext*  ctx = (flags & create_no_freeze) == 0 ? const_cast<SharedContext*>(s.sharedContext()) : 0;
 	WeightConstraint* c = new (m) WeightConstraint(s, ctx, W, rep, sL);
+	if (!c->integrateRoot(s)) {
+		c->destroy(&s, true);
+		return conflict;
+	}
 	if ((flags & create_no_add) == 0) { s.add(c); }
 	return c;
 }
@@ -300,6 +304,37 @@ Constraint* WeightConstraint::cloneAttach(Solver& other) {
 	return new (m) WeightConstraint(other, *this);
 }
 
+bool WeightConstraint::integrateRoot(Solver& s) {
+	if (!s.decisionLevel() || highestUndoLevel(s) >= s.rootLevel() || s.hasConflict()) { return !s.hasConflict(); }
+	// check if constraint has assigned literals
+	uint32 low = s.decisionLevel(), vDL;
+	uint32 np  = 0;
+	for (uint32 i = 0, end = size(); i != end; ++i) {
+		Var v = lits_->var(i);
+		if (s.value(v) != value_free && (vDL = s.level(v)) != 0) { 
+			++np; 
+			s.markSeen(v);
+			low = std::min(low, vDL);
+		}
+	}
+	// propagate assigned literals in assignment order
+	const LitVec& trail = s.trail();
+	const uint32  end   = trail.size() - s.queueSize();
+	GenericWatch* w     = 0;
+	for (uint32 i = s.levelStart(low); i != end && np; ++i) {
+		Literal p = trail[i];
+		if (s.seen(p) && np--) {
+			s.clearSeen(p.var());
+			if (!s.hasConflict() && (w = s.getWatch(trail[i], this)) != 0) {
+				w->propagate(s, p);
+			}
+		}
+	}
+	for (uint32 i = end; i != trail.size() && np; ++i) {
+		if (s.seen(trail[i].var())) { --np; s.clearSeen(trail[i].var()); }
+	}
+	return !s.hasConflict();
+}
 void WeightConstraint::addWatch(Solver& s, uint32 idx, ActiveConstraint c) {
 	// Add watch only if c is relevant.
 	if (uint32(c^1) != active_) {
