@@ -687,7 +687,7 @@ public:
 		solver->force( ~b1, 0 );
 		solver->propagate();
 		CPPUNIT_ASSERT_EQUAL(true, lf->simplify(*solver));
-		CPPUNIT_ASSERT(3u == solver->sharedContext()->numTernary());
+		CPPUNIT_ASSERT(3u == solver->sharedContext()->numLearntShort());
 	}
 
 	void testSimplifyLFRemovesFalseAtoms() {
