@@ -33,6 +33,7 @@ class DecisionHeuristicTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testLookaheadBugNoSimplify);
 	CPPUNIT_TEST(testLookaheadBugDepsNotCleared);
 	CPPUNIT_TEST(testLookaheadBugNoDeps);
+	CPPUNIT_TEST(testLookaheadBugNoNant);
 	CPPUNIT_TEST(testLookaheadStopConflict);
 
 	CPPUNIT_TEST(testBerkmin);
@@ -182,6 +183,24 @@ public:
 		CPPUNIT_ASSERT(lookAtom->select(s));
 		CPPUNIT_ASSERT(s.isFalse(b));
 	}
+	void testLookaheadBugNoNant() {
+		Clasp::Lookahead::Params p(Lookahead::atom_lookahead);
+		p.restrictNant = true;
+		DecisionHeuristic* lookAtom = new UnitHeuristic(p);
+		SharedContext ctx;
+		ctx.master()->heuristic().reset(lookAtom);
+		Literal a = posLit(ctx.addVar(Var_t::atom_var));
+		Literal b = posLit(ctx.addVar(Var_t::atom_var));
+		Literal c = posLit(ctx.addVar(Var_t::atom_var));
+		Solver& s = ctx.startAddConstraints();
+		ctx.addBinary(a, b);
+		ctx.addBinary(b, c);
+		ctx.addBinary(~a, ~b);
+		ctx.addBinary(~b, ~c);
+		ctx.endInit();
+		uint32 n = s.numFreeVars();
+		CPPUNIT_ASSERT(lookAtom->select(s) && s.numFreeVars() != n);
+	}
 	
 	void testLookaheadStopConflict() {
 		DecisionHeuristic* lookAtom = new UnitHeuristic(Lookahead::atom_lookahead);
