@@ -31,6 +31,8 @@ class DefaultMinimizeTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testMultiLevelWeightsAreReused);
 	CPPUNIT_TEST(testMergeComplementaryLits);
 	CPPUNIT_TEST(testMergeComplementaryLits2);
+	CPPUNIT_TEST(testNegativeLowerInit);
+	CPPUNIT_TEST(testNegativeLower);
 
 	CPPUNIT_TEST(testOrder);
 	CPPUNIT_TEST(testSkipLevel);
@@ -212,6 +214,46 @@ public:
 		CPPUNIT_ASSERT(countMinLits() == 1);
 		CPPUNIT_ASSERT(newMin->shared()->adjust(0) == 1 && newMin->shared()->adjust(1) == 0);
 	}
+	void testNegativeLowerInit() {
+		WeightLitVec aMin, bMin;
+		aMin.push_back( WeightLiteral(a, 1) );
+		aMin.push_back( WeightLiteral(b, 1) );
+		bMin.push_back( WeightLiteral(~a, 1) );
+		bMin.push_back( WeightLiteral(~b, 1) );
+		data = MinimizeBuilder()
+			.addRule(aMin)
+			.addRule(bMin)
+			.build(ctx);
+		CPPUNIT_ASSERT(data->lower(0) == 0);
+		CPPUNIT_ASSERT(data->lower(1) == -2);
+	}
+	void testNegativeLower() {
+		ctx.addBinary(a, b);
+		WeightLitVec aMin, bMin;
+		aMin.push_back( WeightLiteral(a, 1) );
+		aMin.push_back( WeightLiteral(b, 1) );
+		bMin.push_back( WeightLiteral(~a, 1) );
+		data = MinimizeBuilder()
+			.addRule(aMin)
+			.addRule(bMin)
+			.build(ctx);
+		newMin = createMin(ctx, *ctx.master(), data, SolverStrategies::opt_hier);
+		newMin->integrateBound(*ctx.master());
+		
+		Solver& s = *ctx.master();
+		s.assume(b) && s.propagate();
+		s.assume(~a) && s.propagate();
+		newMin->handleModel(s);
+		s.undoUntil(0);
+		CPPUNIT_ASSERT(!newMin->integrate(s) || !s.propagate());
+		LitVec ignore;
+		CPPUNIT_ASSERT(newMin->handleUnsat(s, true, ignore));
+		CPPUNIT_ASSERT(newMin->integrate(s) && s.propagate());
+		
+		CPPUNIT_ASSERT(s.isFalse(b));
+		CPPUNIT_ASSERT(s.assume(a) && s.propagate());
+		newMin->handleModel(s);
+	}
 	
 	void testOrder() {
 		WeightLitVec aMin, bMin;
@@ -967,6 +1009,7 @@ class UncoreMinimizeTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testOptimize);
 	CPPUNIT_TEST(testOptimizeGP);
 	CPPUNIT_TEST(testMtBug1);
+	CPPUNIT_TEST(testNegativeLower);
 	CPPUNIT_TEST_SUITE_END(); 
 public:
 	void setUp() {
@@ -1102,6 +1145,37 @@ public:
 		}
 		CPPUNIT_ASSERT(numModels == 1);
 	}
+	void testNegativeLower() {
+		Literal a = posLit(ctx.addVar(Var_t::atom_var));
+		Literal b = posLit(ctx.addVar(Var_t::atom_var));
+		Solver& s = ctx.startAddConstraints();
+		ctx.addBinary(a, b);
+		WeightLitVec aMin, bMin;
+		aMin.push_back( WeightLiteral(a, 1) );
+		aMin.push_back( WeightLiteral(b, 1) );
+		bMin.push_back( WeightLiteral(~a, 1) );
+		data = MinimizeBuilder()
+			.addRule(aMin)
+			.addRule(bMin)
+			.build(ctx);
+		ctx.endInit();
+		data->setMode(MinimizeMode_t::optimize);
+		min = data->attach(s, SolverStrategies::opt_unsat_pre);
+		
+		LitVec ignore;
+		while (!min->integrate(*ctx.master()) && min->handleUnsat(s, true, ignore)) { ; }
+		CPPUNIT_ASSERT(s.assume(~a) && s.propagate());
+		CPPUNIT_ASSERT(s.assume(b) && s.propagate());
+		CPPUNIT_ASSERT(min->handleModel(s));
+		s.undoUntil(0);
+		while (!min->integrate(*ctx.master()) && min->handleUnsat(s, true, ignore)) { ; }
+		CPPUNIT_ASSERT(s.assume(a) && s.propagate());
+		CPPUNIT_ASSERT(s.isFalse(b));
+		min->handleModel(s);
+		s.undoUntil(0);
+		while (!min->integrate(*ctx.master()) && min->handleUnsat(s, true, ignore)) { ; }
+		CPPUNIT_ASSERT(s.hasConflict());
+	}
 private:
 	SharedContext       ctx;
 	SharedMinimizeData* data;
