@@ -151,6 +151,7 @@ class LogicProgramTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testIncrementalRedefine);
 	CPPUNIT_TEST(testIncrementalGetAssumptions);
 	CPPUNIT_TEST(testIncrementalSimplifyCard);
+	CPPUNIT_TEST(testIncrementalSimplifyMinimize);
 	CPPUNIT_TEST_SUITE_END();	
 public:
 	void tearDown(){
@@ -638,8 +639,12 @@ public:
 			.startRule(CHOICERULE).addHead(3).addHead(2).addHead(1).endRule()
 			.startRule().addHead(1).endRule()		// a.
 			.startRule(CONSTRAINTRULE).setBound(1).addHead(2).addToBody(1, true).addToBody(3, true).endRule() // b :- 1 {a, c}.
+			.startRule().addHead(4).addToBody(2, true).addToBody(3, true).endRule()
 		;
+		CPPUNIT_ASSERT(std::distance(builder.getAtom(3)->deps_begin(), builder.getAtom(3)->deps_end()) == 2u);
 		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram() && ctx.endInit());
+		CPPUNIT_ASSERT(std::distance(builder.getAtom(3)->deps_begin(), builder.getAtom(3)->deps_end()) == 1u);
+		CPPUNIT_ASSERT(std::distance(builder.getAtom(2)->deps_begin(), builder.getAtom(2)->deps_end()) == 0u);
 		CPPUNIT_ASSERT_EQUAL(value_free, ctx.master()->value(ctx.symbolTable()[3].lit.var()));
 	}
 	void testDepBodyBug() {
@@ -1597,9 +1602,13 @@ public:
 		;
 		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram() && ctx.endInit());
 		// I2:
-		builder.updateProgram()
-		;
+		builder.updateProgram();
+		builder.setAtomName(3, "c").setAtomName(4, "d");
+		builder.startRule().addHead(3).addToBody(2, true).endRule();
+		builder.startRule().addHead(4).addToBody(2, false).endRule();
 		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram() && ctx.endInit());
+		CPPUNIT_ASSERT_EQUAL(false, builder.getAtom(3)->hasVar());
+		CPPUNIT_ASSERT_EQUAL(posLit(0), builder.getLiteral(4));
 	}
 
 	void testIncrementalBackpropStep() {
@@ -2092,6 +2101,26 @@ public:
 		;
 		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram());
 	}
+
+	void testIncrementalSimplifyMinimize() {
+		builder.start(ctx);
+		// I0:
+		builder.updateProgram();
+		builder.setAtomName(1, "a")
+			.startRule().addHead(1).endRule() // a.
+			.startRule(OPTIMIZERULE).addToBody(1, true).endRule()
+		;
+		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram());
+		CPPUNIT_ASSERT(builder.getMinimizeConstraint()->adjust(0) == 1);
+		builder.disposeMinimizeConstraint();
+		
+		builder.updateProgram();
+		builder.startRule(OPTIMIZERULE).addToBody(1, true).endRule();
+		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram());
+		CPPUNIT_ASSERT(builder.getMinimizeConstraint());
+		CPPUNIT_ASSERT(builder.getMinimizeConstraint()->adjust(0) == 1);
+		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram());
+	}
 private:
 	typedef SharedDependencyGraph DG;
 	SharedContext ctx;
