@@ -69,6 +69,8 @@ class WeightConstraintTest : public CppUnit::TestFixture {
 
 	CPPUNIT_TEST(testAddOnLevel);
 	CPPUNIT_TEST(testCreateSat);
+	CPPUNIT_TEST(testCreateSatOnRoot);
+	CPPUNIT_TEST(testCreateSatOnRootNoProp);
 
 	CPPUNIT_TEST(testMergeNegativeWeight);
 	CPPUNIT_TEST_SUITE_END(); 
@@ -613,6 +615,51 @@ public:
 		}
 		res.first()->destroy(&s, true);
 	}
+	void testCreateSatOnRoot() {
+		ctx.endInit(true);
+		Solver& s = *ctx.master();
+		s.pushRoot(f);
+		CPPUNIT_ASSERT(s.rootLevel() == 1);
+		s.force(a, 0);
+		s.force(b, 0);
+		s.propagate();
+		WeightLitVec lits;
+		lits.push_back(WeightLiteral(a, 1));
+		lits.push_back(WeightLiteral(b, 1));
+		lits.push_back(WeightLiteral(c, 1));
+		lits.push_back(WeightLiteral(d, 1));
+		WeightLitsRep rep           = WeightLitsRep::create(s, lits, 2);
+		WeightConstraint::CPair res = WeightConstraint::create(s, body, rep, WeightConstraint::create_no_add|WeightConstraint::create_sat);
+		CPPUNIT_ASSERT(res.ok() && res.first());
+		CPPUNIT_ASSERT(s.isTrue(body));
+		CPPUNIT_ASSERT(s.reason(body) == res.first());
+		s.popRootLevel(1);
+		CPPUNIT_ASSERT(s.value(body.var()) == value_free);
+		res.first()->destroy(&s, true);
+	}
+	void testCreateSatOnRootNoProp() {
+		ctx.endInit(true);
+		Solver& s = *ctx.master();
+		s.pushRoot(f);
+		CPPUNIT_ASSERT(s.rootLevel() == 1);
+		s.force(a, 0);
+		s.force(b, 0);
+		WeightLitVec lits;
+		lits.push_back(WeightLiteral(a, 1));
+		lits.push_back(WeightLiteral(b, 1));
+		lits.push_back(WeightLiteral(c, 1));
+		lits.push_back(WeightLiteral(d, 1));
+		WeightLitsRep rep           = WeightLitsRep::create(s, lits, 2);
+		WeightConstraint::CPair res = WeightConstraint::create(s, body, rep, WeightConstraint::create_no_add|WeightConstraint::create_sat);
+		CPPUNIT_ASSERT(res.ok() && res.first());
+		CPPUNIT_ASSERT(!s.isTrue(body));
+		CPPUNIT_ASSERT(s.propagate());
+		CPPUNIT_ASSERT(s.isTrue(body));
+		CPPUNIT_ASSERT(s.reason(body) == res.first());
+		s.popRootLevel(1);
+		CPPUNIT_ASSERT(s.value(body.var()) == value_free);
+		res.first()->destroy(&s, true);
+	}
 	void testMergeNegativeWeight() {
 		WeightLitVec lits;
 		lits.push_back(WeightLiteral(a, -1));
